function [X] = samplingDiscrete(n, probabilities)
%SAMPLINGDISCRETE samples from a discrete distribution over the values
% {1,2,...,length(probabilities)} using the rejection sampling method with
% proposal distribution a DiscreteUniform over {1,..,length(probabilities)}
% - n is the number of samples
% - probabilities is a vector of probabilities for the values of x
% For example if probabilities = [p1, p2, p3], \sum_{i=1}^{3}p_i = 1
% then 
%             _  
%            |   p1, k=1
% P(X = k) = |   p2, k=2
%            |_  p3, k=3
% 
% The function returns a sample of X with sample size n 

X = zeros(1,n); % memory allocation for speed
k = length(probabilities);
q = 1/k; % p.m.f of discrete uniform over {1,...,k}
c = max(probabilities ./ q); % optimal c

for i = 1:1:n
    flag = 1;
    while flag
        % sample from the proposal
        y = dunif(k, 1);
        u = rand;
        if (u <= probabilities(y) / (c*q))
            X(i) = y;
            flag = 0;
        end
    end
end

function u = dunif(r, n)
u = zeros(1,n);
u = floor(r.*rand(1,n)) + 1; % return n variates from discrete uniform over {1,...,n}

